import argparse
import os
import toml

from jenesis.config import Config


def run(args: argparse.Namespace):

    project_root = os.path.abspath(os.getcwd())
    contract_root = os.path.join(project_root, "contracts", args.contract)

    # check that we are actually running the command from the project root
    if not os.path.exists(os.path.join(project_root, "jenesis.toml")):
        print("Please run command from project root")
        return

    # check if the contract exists
    if not os.path.exists(contract_root):
        print(f'Contract "{args.contract}" does not exist')
        return

    data = toml.load("jenesis.toml")

    cfg = Config.load(project_root)
    profiles = list(cfg.profiles.keys())

    for profile_name in profiles:
        profile = cfg.profiles[profile_name]
        profile_contracts = list(profile.deployments.keys())

        if args.contract in profile_contracts:
            contract_data = data["profile"][profile_name]["contracts"][args.contract].copy()
            data["profile"][profile_name]["contracts"][args.deployment] = contract_data
            data["profile"][profile_name]["contracts"][args.deployment]["name"] = args.deployment

    project_configuration_file = os.path.join(project_root, "jenesis.toml")

    with open(project_configuration_file, "w", encoding="utf-8") as toml_file:
        toml.dump(data, toml_file)


def add_deployment_command(parser):
    deployment_cmd = parser.add_parser("deployment")
    deployment_cmd.add_argument("contract", help="The name of the contract to duplicate")
    deployment_cmd.add_argument("deployment", help="The name to assign the new deployment")
    deployment_cmd.set_defaults(handler=run)



import os.path
from dataclasses import dataclass
from typing import Optional

from cosmpy.aerial.contract import _compute_digest


@dataclass
class Contract:
    name: str
    source_path: str
    binary_path: str
    cargo_root: str
    schema: dict

    def __post_init__(self):
        self.instantiate_schema = {}
        self.query_schema = {}
        self.execute_schema = {}

        # check for workspace-style schemas
        if self.name in self.schema:
            contract_schema = self.schema[self.name]
        else:
            contract_schema = self.schema

        for (msg_type, schema) in contract_schema.items():
            if 'instantiate' in msg_type:
                self.instantiate_schema = schema
            elif 'query' in msg_type:
                self.query_schema = schema
            elif 'execute' in msg_type:
                self.execute_schema = schema

    def digest(self) -> Optional[str]:
        if not os.path.isfile(self.binary_path):
            return None

        return _compute_digest(self.binary_path).hex()

    def execute_msgs(self) -> dict:
        return self._extract_msgs(self.execute_schema)

    def query_msgs(self) -> dict:
        return self._extract_msgs(self.query_schema)

    def init_args(self) -> dict:
        return list(self.instantiate_schema.get('properties',{}).keys())

    def _extract_msgs(self, msg_schema: dict) -> dict:
        msgs = {}
        if 'oneOf' in msg_schema:
            schemas = msg_schema['oneOf']
        elif 'anyOf' in msg_schema:
            schemas = msg_schema['anyOf']
        else:
            return msgs
        for schema in schemas:
            msg = schema['required'][0]
            if 'required' in schema['properties'][msg]:
                args = schema['properties'][msg]['required']
            else:
                args = []
            msgs[msg] = args
        return msgs

    def __repr__(self):
        return self.name
